@@ -2,11 +2,12 @@
  * LUKS - Linux Unified Key Setup
  *
  * Copyright (C) 2004-2006, Clemens Fruhwirth <clemens@endorphin.org>
- * Copyright (C) 2009-2012, Red Hat, Inc. All rights reserved.
+ * Copyright (C) 2009-2018, Red Hat, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
- * version 2 as published by the Free Software Foundation.
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -39,6 +40,9 @@
 #define LUKS_MKD_ITERATIONS_MIN  1000
 #define LUKS_SLOT_ITERATIONS_MIN 1000
 
+// Iteration time for digest in ms
+#define LUKS_MKD_ITERATIONS_MS 125
+
 #define LUKS_KEY_DISABLED_OLD 0
 #define LUKS_KEY_ENABLED_OLD 0xCAFE
 
@@ -105,8 +109,6 @@ int LUKS_generate_phdr(
 	unsigned int stripes,
 	unsigned int alignPayload,
 	unsigned int alignOffset,
-	uint32_t iteration_time_ms,
-	uint64_t *PBKDF2_per_sec,
 	int detached_metadata_device,
 	struct crypt_device *ctx);
 
@@ -129,7 +131,6 @@ int LUKS_hdr_uuid_set(
 
 int LUKS_hdr_backup(
 	const char *backup_file,
-	struct luks_phdr *hdr,
 	struct crypt_device *ctx);
 
 int LUKS_hdr_restore(
@@ -147,8 +148,6 @@ int LUKS_set_key(
 	size_t passwordLen,
 	struct luks_phdr *hdr,
 	struct volume_key *vk,
-	uint32_t iteration_time_ms,
-	uint64_t *PBKDF2_per_sec,
 	struct crypt_device *ctx);
 
 int LUKS_open_key_with_hdr(
@@ -168,24 +167,12 @@ crypt_keyslot_info LUKS_keyslot_info(struct luks_phdr *hdr, int keyslot);
 int LUKS_keyslot_find_empty(struct luks_phdr *hdr);
 int LUKS_keyslot_active_count(struct luks_phdr *hdr);
 int LUKS_keyslot_set(struct luks_phdr *hdr, int keyslot, int enable);
-int LUKS_keyslot_area(struct luks_phdr *hdr,
+int LUKS_keyslot_area(const struct luks_phdr *hdr,
 	int keyslot,
 	uint64_t *offset,
 	uint64_t *length);
-
-int LUKS_encrypt_to_storage(
-	char *src, size_t srcLength,
-	struct luks_phdr *hdr,
-	struct volume_key *vk,
-	unsigned int sector,
-	struct crypt_device *ctx);
-
-int LUKS_decrypt_from_storage(
-	char *dst, size_t dstLength,
-	struct luks_phdr *hdr,
-	struct volume_key *vk,
-	unsigned int sector,
-	struct crypt_device *ctx);
+size_t LUKS_device_sectors(const struct luks_phdr *hdr);
+size_t LUKS_keyslots_offset(const struct luks_phdr *hdr);
 
 int LUKS1_activate(struct crypt_device *cd,
 		   const char *name,
