@@ -1,11 +1,13 @@
 /*
  * loopback block device utilities
  *
- * Copyright (C) 2011-2012, Red Hat, Inc. All rights reserved.
+ * Copyright (C) 2009-2018, Red Hat, Inc. All rights reserved.
+ * Copyright (C) 2009-2018, Milan Broz
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
- * version 2 as published by the Free Software Foundation.
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -17,6 +19,7 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
+#include <stdlib.h>
 #include <string.h>
 #include <stdio.h>
 #include <unistd.h>
@@ -25,6 +28,10 @@
 #include <limits.h>
 #include <sys/ioctl.h>
 #include <sys/stat.h>
+#include <sys/types.h>
+#ifdef HAVE_SYS_SYSMACROS_H
+# include <sys/sysmacros.h>     /* for major, minor */
+#endif
 #include <linux/loop.h>
 
 #include "utils_loop.h"
@@ -39,17 +46,18 @@
 #define LOOP_CTL_GET_FREE 0x4C82
 #endif
 
+#ifndef LOOP_SET_CAPACITY
+#define LOOP_SET_CAPACITY 0x4C07
+#endif
+
 static char *crypt_loop_get_device_old(void)
 {
 	char dev[20];
 	int i, loop_fd;
-	struct stat st;
 	struct loop_info64 lo64 = {0};
 
 	for (i = 0; i < 256; i++) {
 		sprintf(dev, "/dev/loop%d", i);
-		if (stat(dev, &st) || !S_ISBLK(st.st_mode))
-			return NULL;
 
 		loop_fd = open(dev, O_RDONLY);
 		if (loop_fd < 0)
@@ -66,7 +74,7 @@ static char *crypt_loop_get_device_old(void)
 	return NULL;
 }
 
-char *crypt_loop_get_device(void)
+static char *crypt_loop_get_device(void)
 {
 	char dev[64];
 	int i, loop_fd;
@@ -92,32 +100,50 @@ char *crypt_loop_get_device(void)
 	return strdup(dev);
 }
 
-int crypt_loop_attach(const char *loop, const char *file, int offset,
+int crypt_loop_attach(char **loop, const char *file, int offset,
 		      int autoclear, int *readonly)
 {
 	struct loop_info64 lo64 = {0};
+	char *lo_file_name;
 	int loop_fd = -1, file_fd = -1, r = 1;
 
+	*loop = NULL;
+
 	file_fd = open(file, (*readonly ? O_RDONLY : O_RDWR) | O_EXCL);
-	if (file_fd < 0 && errno == EROFS && !*readonly) {
+	if (file_fd < 0 && (errno == EROFS || errno == EACCES) && !*readonly) {
 		*readonly = 1;
 		file_fd = open(file, O_RDONLY | O_EXCL);
 	}
 	if (file_fd < 0)
 		goto out;
 
-	loop_fd = open(loop, *readonly ? O_RDONLY : O_RDWR);
-	if (loop_fd < 0)
-		goto out;
+	while (loop_fd < 0)  {
+		*loop = crypt_loop_get_device();
+		if (!*loop)
+			goto out;
 
-	strncpy((char*)lo64.lo_file_name, file, LO_NAME_SIZE);
+		loop_fd = open(*loop, *readonly ? O_RDONLY : O_RDWR);
+		if (loop_fd < 0)
+			goto out;
+
+		if (ioctl(loop_fd, LOOP_SET_FD, file_fd) < 0) {
+			if (errno != EBUSY)
+				goto out;
+			free(*loop);
+			*loop = NULL;
+
+			close(loop_fd);
+			loop_fd = -1;
+		}
+	}
+
+	lo_file_name = (char*)lo64.lo_file_name;
+	lo_file_name[LO_NAME_SIZE-1] = '\0';
+	strncpy(lo_file_name, file, LO_NAME_SIZE-1);
 	lo64.lo_offset = offset;
 	if (autoclear)
 		lo64.lo_flags |= LO_FLAGS_AUTOCLEAR;
 
-	if (ioctl(loop_fd, LOOP_SET_FD, file_fd) < 0)
-		goto out;
-
 	if (ioctl(loop_fd, LOOP_SET_STATUS64, &lo64) < 0) {
 		(void)ioctl(loop_fd, LOOP_CLR_FD, 0);
 		goto out;
@@ -139,6 +165,10 @@ out:
 		close(loop_fd);
 	if (file_fd >= 0)
 		close(file_fd);
+	if (r && *loop) {
+		free(*loop);
+		*loop = NULL;
+	}
 	return r ? -1 : loop_fd;
 }
 
@@ -157,6 +187,21 @@ int crypt_loop_detach(const char *loop)
 	return r;
 }
 
+int crypt_loop_resize(const char *loop)
+{
+	int loop_fd = -1, r = 1;
+
+	loop_fd = open(loop, O_RDONLY);
+	if (loop_fd < 0)
+                return 1;
+
+	if (!ioctl(loop_fd, LOOP_SET_CAPACITY, 0))
+		r = 0;
+
+	close(loop_fd);
+	return r;
+}
+
 static char *_ioctl_backing_file(const char *loop)
 {
 	struct loop_info64 lo64 = {0};
